@@ -1,36 +1,42 @@
-from functools import partial
-from signal import SIGTERM
-import time
+import multiprocessing
 import os
 import sys
 import threading
+import cherrypy  # type: ignore
+import time
+from functools import partial
+from queue import Queue
+from signal import SIGTERM
+from typing import List, Dict, Type, Optional, Any
+
+import requests
+
 import resotolib.proc
-from typing import List, Dict, Type, Optional, Any, cast
-from resotoworker.config import add_config
-from resotolib.config import Config
-from resotolib.logger import log, setup_logger, add_args as logging_add_args
-from resotolib.jwt import add_args as jwt_add_args
-from resotolib.baseplugin import BaseActionPlugin, BasePostCollectPlugin, BaseCollectorPlugin, PluginType
-from resotolib.web import WebServer
-from resotolib.web.metrics import WebApp
-from resotolib.proc import log_stats, increase_limits
 from resotolib.args import ArgumentParser
+from resotolib.baseplugin import BaseActionPlugin, BasePostCollectPlugin, BaseCollectorPlugin, PluginType
+from resotolib.config import Config
 from resotolib.core import add_args as core_add_args, resotocore, wait_for_resotocore
+from resotolib.core.actions import CoreActions, CoreFeedback
 from resotolib.core.ca import TLSData
-from resotolib.core.actions import CoreActions
-from resotolib.core.tasks import CoreTasks
-from resotoworker.pluginloader import PluginLoader
-from resotoworker.collect import Collector
-from resotoworker.cleanup import cleanup
-from resotoworker.tag import core_tag_tasks_processor
+from resotolib.core.tasks import CoreTasks, CoreTaskHandler
 from resotolib.event import (
     add_event_listener,
     Event,
     EventType,
 )
+from resotolib.jwt import add_args as jwt_add_args
+from resotolib.logger import log, setup_logger, add_args as logging_add_args
+from resotolib.core.custom_command import command_definitions
+from resotolib.proc import log_stats, increase_limits
+from resotolib.types import Json
+from resotolib.web import WebServer
+from resotolib.web.metrics import WebApp
+from resotoworker.cleanup import cleanup
+from resotoworker.collect import Collector
+from resotoworker.config import add_config
+from resotoworker.pluginloader import PluginLoader
 from resotoworker.resotocore import Resotocore
-import requests
-
+from resotoworker.tag import core_tag_tasks_processor
 
 # This will be used in main() and shutdown()
 shutdown_event = threading.Event()
@@ -99,7 +105,11 @@ def main() -> None:
 
     core = Resotocore(send_request, config)
 
-    collector = Collector(core.send_to_resotocore, config)
+    # the multiprocessing manager is used to share data between processes
+    mp_manager = multiprocessing.Manager()
+    core_messages: Queue[Json] = mp_manager.Queue()
+
+    collector = Collector(config, core.send_to_resotocore, core_messages)
 
     # Handle Ctrl+c and other means of termination/shutdown
     resotolib.proc.initializer()
@@ -109,13 +119,13 @@ def main() -> None:
     increase_limits()
 
     web_server_args = {}
-    if tls_data:
+    if tls_data and not Config.resotoworker.no_tls:
         web_server_args = {
             "ssl_cert": tls_data.cert_path,
             "ssl_key": tls_data.key_path,
         }
     web_server = WebServer(
-        WebApp(mountpoint=Config.resotoworker.web_path),
+        WorkerWebApp(mountpoint=Config.resotoworker.web_path, plugin_loader=plugin_loader),
         web_host=Config.resotoworker.web_host,
         web_port=Config.resotoworker.web_port,
         **web_server_args,
@@ -139,31 +149,44 @@ def main() -> None:
         },
         message_processor=partial(core_actions_processor, config, plugin_loader, tls_data, collector),
         tls_data=tls_data,
+        incoming_messages=core_messages,
     )
 
-    task_queue_filter = {}
-    if len(Config.resotoworker.collector) > 0:
-        task_queue_filter = {"cloud": list(Config.resotoworker.collector)}
+    # make tagging by collectors available out of the box
+    collect_plugins: List[BaseCollectorPlugin] = plugin_loader.plugins(PluginType.COLLECTOR)  # type: ignore
+    task_handler = [
+        CoreTaskHandler(
+            name="tag",
+            info="already provided",
+            description="already provided",
+            filter={"cloud": [plugin.cloud]},
+            expect_node_result=True,
+            handler=partial(core_tag_tasks_processor, plugin, config),
+        )
+        for plugin in collect_plugins
+    ]
+    # search all other plugins for possible task providers
+    for plugin_clazz in plugin_loader.all_plugins():
+        plugin = plugin_clazz()
+        for wtd in command_definitions(plugin_clazz):
+            handler = CoreTaskHandler.from_definition(plugin, wtd)
+            log.info(f"Plugin {plugin.name}: Add task handler for task {handler.name} @ {handler.filter}")
+            task_handler.append(handler)
+
     core_tasks = CoreTasks(
-        identifier=f"{ArgumentParser.args.subscriber_id}-tagger",
+        identifier=f"{ArgumentParser.args.subscriber_id}-task-handler",
         resotocore_ws_uri=resotocore.ws_uri,
-        tasks=["tag"],
-        task_queue_filter=task_queue_filter,
-        message_processor=partial(
-            core_tag_tasks_processor,
-            {p.cloud: p for p in cast(List[Type[BaseCollectorPlugin]], plugin_loader.plugins(PluginType.COLLECTOR))},
-            config,
-        ),
+        task_handler=task_handler,
         tls_data=tls_data,
     )
     core_actions.start()
     core_tasks.start()
 
-    for Plugin in plugin_loader.plugins(PluginType.ACTION):
-        assert issubclass(Plugin, BaseActionPlugin)
+    for plugin_class in plugin_loader.plugins(PluginType.ACTION):
+        assert issubclass(plugin_class, BaseActionPlugin)
         try:
-            log.debug(f"Starting action plugin {Plugin}")
-            plugin = Plugin(tls_data=tls_data)
+            log.debug(f"Starting action plugin {plugin_class}")
+            plugin = plugin_class(tls_data=tls_data)
             plugin.start()
         except Exception as e:
             log.exception(f"Caught unhandled persistent Plugin exception {e}")
@@ -193,14 +216,15 @@ def core_actions_processor(
         return None
     kind = message.get("kind")
     message_type = message.get("message_type")
-    data = message.get("data")
-    task_id = data.get("task")  # type: ignore
+    data = message.get("data") or {}
+    task_id: str = data.get("task")  # type: ignore
+    step_name: str = data.get("step")  # type: ignore
     log.debug(f"Received message of kind {kind}, type {message_type}, data: {data}")
     if kind == "action":
         try:
             if message_type == "collect":
                 start_time = time.time()
-                collector.collect_and_send(collectors, post_collectors, task_id=task_id)
+                collector.collect_and_send(collectors, post_collectors, task_id=task_id, step_name=step_name)
                 run_time = int(time.time() - start_time)
                 log.info(f"Collect ran for {run_time} seconds")
             elif message_type == "cleanup":
@@ -210,9 +234,11 @@ def core_actions_processor(
                     if Config.resotoworker.cleanup_dry_run:
                         log.info("Cleanup called with dry run configured" " (resotoworker.cleanup_dry_run)")
                     start_time = time.time()
+                    feedback = CoreFeedback(task_id, step_name, "cleanup", collector.core_messages)
                     cleanup(
                         config,
                         {p.cloud: p for p in collectors},
+                        feedback,
                         tls_data=tls_data,
                     )
                     run_time = int(time.time() - start_time)
@@ -220,7 +246,9 @@ def core_actions_processor(
             else:
                 raise ValueError(f"Unknown message type {message_type}")
         except Exception as e:
-            log.exception(f"Failed to {message_type}: {e}")
+            msg = f"Failed to {message_type}: {e}"
+            data["error"] = msg
+            log.exception(msg)
             reply_kind = "action_error"
         else:
             reply_kind = "action_done"
@@ -268,5 +296,23 @@ def add_args(arg_parser: ArgumentParser) -> None:
     )
 
 
+class WorkerWebApp(WebApp):
+    def __init__(self, *args, plugin_loader: PluginLoader, **kwargs) -> None:  # type: ignore
+        super().__init__(*args, **kwargs)
+        self.plugin_loader = plugin_loader
+
+    @cherrypy.expose  # type: ignore
+    @cherrypy.tools.json_out()  # type: ignore
+    @cherrypy.tools.allow(methods=["GET"])  # type: ignore
+    def info(self) -> Dict[str, List[str]]:
+        active_collectors: List[str] = [
+            plugin.cloud for plugin in self.plugin_loader.plugins(PluginType.COLLECTOR)  # type: ignore
+        ]
+        all_collectors: List[str] = [
+            plugin.cloud for plugin in self.plugin_loader.all_plugins(PluginType.COLLECTOR)  # type: ignore
+        ]
+        return {"active_collectors": active_collectors, "all_collectors": all_collectors}
+
+
 if __name__ == "__main__":
     main()
